@@ -3,24 +3,21 @@ package App::cpanminus::reporter;
 use warnings;
 use strict;
 
-our $VERSION = '0.06';
+our $VERSION = '0.07';
 
 use Carp ();
 use File::Spec     3.19;
 use File::HomeDir  0.58 ();
 use Test::Reporter 1.54;
-use CPAN::Testers::Common::Client;
+use CPAN::Testers::Common::Client 0.04;
 use CPAN::Testers::Common::Client::Config;
 use Parse::CPAN::Meta;
 use CPAN::Meta::Converter;
 use Try::Tiny;
 use URI;
 use Metabase::Resource;
-use File::stat;
 use Capture::Tiny qw(capture);
-
-# TODO: factor these into CPAN::Testers::Common::Client?
-use Config::Tiny 2.08 ();
+use IO::Prompt::Tiny ();
 
 sub new {
   my ($class, %params) = @_;
@@ -28,51 +25,34 @@ sub new {
 
   $self->quiet( $params{quiet} );
 
-  my $config = CPAN::Testers::Common::Client::Config->new;
-  my $config_filename = $config->get_config_filename();
-  my $config_data = Config::Tiny->read( $config_filename );
-
-  # FIXME: poor man's validation, we should factor this out
-  # from CPAN::Reporter::Config SOON!
-  #FIXME: currently, this only cares for email_from and transport.
-  unless ($config_data) {
-    warn "Error reading configuration file '$config_filename': "
-      . Config::Tiny->errstr() . "\nFalling back to default values\n"
-          unless $self->quiet;
-
-    $config = {
-      _ => {
-        edit_report => 'default:no pass/na:no',
-        email_from  => getpwuid($<) . '@localhost',
-        send_report => 'default:yes pass/na:yes',
-        transport   => 'Metabase uri https://metabase.cpantesters.org/api/v1/ id_file metabase_id.json',
-      },
-    };
+  my $config = CPAN::Testers::Common::Client::Config->new(
+    prompt => \&IO::Prompt::Tiny::prompt,
+  );
+  if ($params{setup}) {
+    $config->setup;
+    return;
   }
 
-  my @transport = split /\s+/ => $config_data->{_}{transport};
-  my $transport_name = shift @transport
-    or die 'transport method missing.';
+  my $config_filename = $config->get_config_filename;
+  if ( -e $config_filename ) {
+    if ( !$config->read ) {
+      print "Error reading CPAN Testers configuration file '$config_filename'. Aborting.";
+      return;
+    }
+  }
+  else {
+    my $answer = IO::Prompt::Tiny::prompt("CPAN Testers configuration file '$config_filename' not found. Would you like to set it up now? (y/n)", 'y');
 
-  # unlike other transports, Metabase uses its args as a hash
-  # and forces us to normalize the given id_file.
-  if ($transport_name eq 'Metabase') {
-    my %transport_args = @transport;
-    $transport_args{id_file} = $config->normalize_id_file( $transport_args{id_file} );
-    @transport = %transport_args;
-
-    if ( ! -r $transport_args{id_file} ) {
-      die "Error loading Metabase transport 'id_file' file at '$transport_args{id_file}'\n";
+    if ( $answer =~ /^y/i ) {
+      $config->setup;
+    }
+    else {
+      print "The CPAN Testers configuration file is required. Aborting.\n";
+      return;
     }
   }
 
-  $config_data->{_}{transport} = {
-    name => $transport_name,
-    args => [ @transport ],
-  };
-
-  $config_data->{_}{email_from} = $params{email_from} if exists $params{email_from};
-  $self->config( $config_data->{_} );
+  $self->config( $config );
 
   $self->build_dir(
     $params{build_dir}
@@ -87,8 +67,8 @@ sub new {
   # as a safety mechanism, we only let people parse build.log files
   # if they were generated up to 30 minutes (1800 seconds) ago,
   # unless the user asks us to --force it.
-  my $st = lstat $self->build_logfile;
-  if ( !$params{force} && time - $st->mtime > 1800 ) {
+  my $mtime = (stat $self->build_logfile)[9];
+  if ( !$params{force} && $mtime && time - $mtime > 1800 ) {
       die <<'EOMESSAGE';
 Fatal: build.log was created longer than 30 minutes ago.
 
@@ -110,7 +90,6 @@ EOMESSAGE
   return $self;
 }
 
-
 ## basic accessors ##
 
 sub config {
@@ -155,6 +134,12 @@ sub run {
   open my $fh, '<', $logfile
     or Carp::croak "error opening build log file '$logfile' for reading: $!";
 
+  my $header = <$fh>;
+  if ($header =~ /^cpanm \(App::cpanminus\) (\d+\.\d+) on perl (\d+\.\d+)/) {
+    $self->{_cpanminus_version} = $1;
+    $self->{_perl_version} = $2;
+  }
+
   my $found = 0;
   my $parser;
 
@@ -248,15 +233,9 @@ sub make_report {
     return;
   }
 
-  ## NOTE: Whenever cpanm is called, it resets build.log
-  ##       This is an interesting side-effect that helps us
-  ##       to refrain from sending duplicate reports.
-  my $cpanm_version = capture { system('cpanm -V') };
-  chomp $cpanm_version;
-  $cpanm_version = 'unknown cpanm' unless $cpanm_version =~ /\d+/;
-
   print "sending: ($resource, $author, $dist, $result)\n" unless $self->quiet;
 
+  my $cpanm_version = $self->{_cpanminus_version} || 'unknown cpanm version';
   my $meta = $self->get_meta_for( $dist );
   my $client = CPAN::Testers::Common::Client->new(
     author      => $author,
@@ -269,12 +248,12 @@ sub make_report {
 
   my $dist_file = join '/' => ($uri->path_segments)[-2,-1];
   my $reporter = Test::Reporter->new(
-    transport      => $self->config->{transport}{name},
-    transport_args => $self->config->{transport}{args},
+    transport      => $self->config->transport_name,
+    transport_args => $self->config->transport_args,
     grade          => $client->grade,
     distribution   => $dist,
     distfile       => $dist_file,
-    from           => $self->config->{email_from},
+    from           => $self->config->email_from,
     comments       => $client->email,
     via            => $client->via,
   );
